# vim: set fileencoding=ascii ts=4 sw=4 sts=0 :
#
# ZShogi.py
#
# Copyright (C) 2004, Shu KONNO <owa@bg.wakwak.com> All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsuo Ishimoto not be used in advertising or publicity pertaining
# to distribution of the software without specific, written prior permission.
#
# SHU KONNO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
#
##############################################################################
"""ZShogi: zope japanese shogi product."""

__version__='$Revision: 0.1.1.0 $'[11:-2]

from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem

from board9 import opening_stage_labels
from board9.CntlSpace import ViewSpace, VCOpenSpace, VCMakeSpace, VCPlaySpace
from board9.CntlSpace import COpenSpace, CMakeSpace, CPlaySpace
from ZEvent import proc_view_event, proc_view_prev, proc_view_next
from ZEvent import proc_edit_event, proc_init_event, proc_move_event
import ZView
import re

dtml_add = DTMLFile('dtml/ManageAdd', globals())

def manage_add(self, id, title='', REQUEST=None):
	"""Method to add a new ZShogi object."""
	newobj = ZShogi(id, title)
	self._setObject(id, newobj)
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)


class ZShogi(PropertyManager, SimpleItem):
	"""ZShogi Class"""

	meta_type = "ZShogi"

	# create a ClassSecurityInfo object
	security = ClassSecurityInfo()
	security.setPermissionDefault('View',('Anonymous','Manager',))
	#security.setDefaultAccess("deny")

	manage_options = (
			( {'label':'Edit', 'action':'manage_edit'}
			, {'label':'Init', 'action':'manage_init'}
			, {'label':'Move', 'action':'manage_move'}
			, {'label':'View', 'action':'manage_view'}
			)
			+ PropertyManager.manage_options
			+ SimpleItem.manage_options
		)

	security.declareProtected('View', 'manage_main')
	security.declareProtected('View', 'manage_view')
	security.declareProtected('Manage ZShogi', 'manage_edit')
	security.declareProtected('Manage ZShogi', 'manage_init')
	security.declareProtected('Manage ZShogi', 'manage_move')
	security.declareProtected('Manage ZShogi', 'manage_import')

	# DTML Method
	dtml_edit = DTMLFile('dtml/ManageEdit', globals())
	dtml_init = DTMLFile('dtml/ManageInit', globals())
	dtml_move = DTMLFile('dtml/ManageMove', globals())
	dtml_view = DTMLFile('dtml/ManageView', globals())
	dtml_main = DTMLFile('dtml/ManageMain', globals())

	dtml_suite_puzzle = DTMLFile('dtml/SuitePuzzle', globals())

	security.declarePrivate('__init__')
	def __init__(self, id, title=''):
		"""Method to initialize ZShogi instance."""
		self.id = id;
		self.title = title
		self._type = opening_stage_labels[2][0]
		self._diag = self._type
		self._side = 1
		self._hist = ''
		self._numb = 0
		self._nmax = 0
		self._memo = {}

	def getMemo(self, n=None):
		if not n:
			n = self._numb
		return self._memo.get(int(n), '')

	def getType(self):
		return self._type

	def nmax(self):
		return self._nmax

	def setTypeDiag(self, type, diag):
		self._type = type
		self._diag = diag

	def setScore(self, hist, side=1):
		self._side = side
		self._hist = hist
		n = 0
		if hist:
			n = len(hist.split(','))
		self._numb = n
		self._nmax = n

	def index_html(self, REQUEST=None):
		"""The default view of the content of ZShogi."""
		if REQUEST is not None:
			if not REQUEST.form:
				REQUEST['sg_diag'] = self._diag
				REQUEST['sg_hist'] = self._hist
				REQUEST['sg_numb'] = self._numb
				REQUEST['sg_nmax'] = self._nmax
			else:
				proc_view_event(REQUEST)
			REQUEST.SESSION['ss_diag'] = REQUEST['sg_diag']
			return self.dtml_main(self, REQUEST)

	def manage_view(self, REQUEST=None):
		"""Method to view ZShogi instance."""
		if REQUEST is not None:
			if not REQUEST.form:
				REQUEST['sg_diag'] = self._diag
				REQUEST['sg_hist'] = self._hist
				REQUEST['sg_numb'] = self._numb
				REQUEST['sg_nmax'] = self._nmax
			else:
				proc_view_event(REQUEST)
			REQUEST.SESSION['ss_diag'] = REQUEST['sg_diag']
			return self.dtml_view(self, REQUEST)

	def manage_edit(self, REQUEST=None):
		"""Method to edit ZShogi instance."""
		if REQUEST is not None:
			if not REQUEST.form:
				REQUEST['sg_title'] = self.title
				REQUEST['sg_diag'] = self._diag
				REQUEST['sg_hist'] = self._hist
				REQUEST['sg_numb'] = self._numb
				REQUEST['sg_nmax'] = self._nmax
			elif REQUEST.has_key('update'):
				self.title = REQUEST['sg_title']
				self._diag = REQUEST['sg_diag']
				self._hist = REQUEST['sg_hist']
				self._numb = int(REQUEST['sg_numb'])
				self._nmax = int(REQUEST['sg_nmax'])
				old_memo = self._memo.get(self._numb, '')
				new_memo = REQUEST['sg_memo']
				if old_memo or new_memo:
					self._memo[self._numb] = new_memo
			elif REQUEST.has_key('import'):
				self.manage_import(REQUEST)
			else:
				proc_edit_event(REQUEST)

			REQUEST['sg_memo'] = self._memo.get(int(REQUEST.sg_numb), '')
			REQUEST.SESSION.set('ss_diag', REQUEST['sg_diag'])
			return self.dtml_edit(self, REQUEST)

	def manage_import(self, REQUEST):
		file = REQUEST.get('file', None)
		if file:
			hist = file.read()
			hist = re.sub('\s', '', hist)
			REQUEST['sg_diag'] = self._type
			REQUEST['sg_hist'] = hist
			REQUEST['sg_numb'] = 0
			REQUEST['sg_nmax'] = len(hist.split(','))
			proc_view_next(REQUEST)

	def manage_init(self, REQUEST=None):
		"""Method to initialize ZShogi instance."""
		if REQUEST is not None:
			if not REQUEST.form:
				REQUEST['sg_type'] = self._type
				REQUEST['sg_diag'] = self._diag
				REQUEST['sg_side'] = self._side
				REQUEST['sg_hist'] = self._hist
				REQUEST['sg_pick'] = ''
				REQUEST['sg_numb'] = self._numb
				proc_view_prev(REQUEST)
			elif REQUEST.has_key('update'):
				self._type = REQUEST['sg_type']
				self._diag = re.sub('\s', '', REQUEST['sg_diag'])
				self._side = int(REQUEST['sg_side'])
				self._hist = ''
				self._numb = 0
				self._nmax = 0
				self._memo = {}
			else:
				proc_init_event(REQUEST, size=0)

			REQUEST.set('opening', opening_stage_labels)
			REQUEST.SESSION.set('ss_diag', REQUEST['sg_diag'])
			REQUEST.SESSION.set('ss_pick', REQUEST['sg_pick'])
			REQUEST.set('sg_diag', re.sub(',', ', ', REQUEST['sg_diag']))
			return self.dtml_init(self, REQUEST)

	def manage_move(self, REQUEST=None):
		"""Method to move ZShogi instance."""
		if REQUEST is not None:
			if not REQUEST.form:
				REQUEST.set('sg_type', self._type)
				REQUEST.set('sg_diag', self._diag)
				REQUEST.set('sg_side', self._side)
				REQUEST.set('sg_hist', self._hist)
				REQUEST.set('sg_pick', '')
				REQUEST.set('sg_numb', self._numb)
				proc_view_next(REQUEST)
			elif REQUEST.has_key('update'):
				self._diag = REQUEST.get('sg_diag', self._diag)
				self._hist = REQUEST.get('sg_hist', self._hist)
				self._numb = len(self._hist.split(','))
				self._nmax = self._numb
			else:
				proc_move_event(REQUEST, size=0)

			REQUEST.SESSION.set('ss_diag', REQUEST.get('sg_diag', ''))
			REQUEST.SESSION.set('ss_pick', REQUEST.get('sg_pick', ''))
			return self.dtml_move(self, REQUEST)

	security.declareProtected('View', 'get_script')
	def get_script(self, REQUEST):
		"""Method to get scripts which describe game progress."""
		hist = REQUEST.get('sg_hist', '')
		if hist:
			list = hist.split(',')
			return list[int(REQUEST.sg_numb) - 1]
		return ''

	security.declareProtected('View', 'get_scripts')
	def get_scripts(self, REQUEST):
		"""Method to get scripts which describe game progress."""
		hist = REQUEST.get('sg_hist', '')
		if hist:
			return hist.split(',')
		return None

	security.declareProtected('View', 'get_player')
	def get_player(self, REQUEST):
		"""Method to get player to be able to play in this ocation."""
		s = ''
		num = int(self._side)
		hist = REQUEST.get('sg_hist', '')
		if hist:
			num += len(hist.split(','))
		if num % 2:
			s = 'black:'
		else:
			s = 'white:'
		pick = REQUEST.get('sg_pick', '')
		if pick:
			s += pick[0] + pick[1]
			if len(pick) > 2 and pick[2]:
				s += pick[2]
		return s

	#### methods to display ###
	security.declareProtected('View', 'getImage')
	def getImage(self, REQUEST=None, size=0):
		"""Method to get ZShogi image."""
		if REQUEST is not None:
			REQUEST.RESPONSE.setHeader('Content-Type','image/png')
		view = ViewSpace(diagram=self._diag, size=int(size))
		#"Content-type: image/png\n\n"
		if self._type == 'S0':
			return view.getPuzzleImage()
		else:
			return view.getWholeImage()

	security.declareProtected('View', 'showDiag')
	def showDiag(self, REQUEST, size=0):
		"""Method to show ZShogi instance."""
		return ZView.showCurrent(REQUEST, size)

	security.declareProtected('View', 'showBoard')
	def showBoard(self, REQUEST, size=0):
		"""Method to show board image."""
		return ZView.showBoard(REQUEST, size)

	security.declareProtected('View', 'showBlack')
	def showBlack(self, REQUEST, size=0):
		"""Method to show blackate image of blacks."""
		return ZView.showBlacket1(REQUEST, size)

	security.declareProtected('View', 'showWhite')
	def showWhite(self, REQUEST, size=0):
		"""Method to show blackate image of whites."""
		return ZView.showBlacket2(REQUEST, size)

	#### tools to display ###
	security.declareProtected('View', 'show')
	def show(self, seq='', size=0):
		"""Method to show ZShogi instance."""
		view = VCPlaySpace(self._diag, self._hist, self._numb, size=int(size))
		if seq:
			seq = int(seq)
			if seq > self._numb:
				view.next(seq - self._numb)
			elif self._numb > seq:
				view.prev(self._numb - seq)
		if self._type == 'S0':
			return view.getPuzzleImage()
		else:
			return view.getWholeImage()

	security.declareProtected('View', 'showChange')
	def showChange(self, REQUEST=None):
		"""Method to get next sequence number."""
		seq = self._numb
		if REQUEST is not None:
			if REQUEST.form:
				form = REQUEST.form
				seq = int(form.get('seq', self._numb))
				if form.has_key('se_head'):
					seq = 0
				elif form.has_key('se_prev'):
					if seq > 0:
						seq -= 1
				elif form.has_key('se_next'):
					if seq < self._nmax:
						seq += 1
				elif form.has_key('se_tail'):
					seq = self._nmax
			REQUEST.set('seq', seq)
		return str(seq)

	security.declareProtected('View', 'showCntl')
	def showCntl(self, REQUEST=None, style='border-width:1;'):
		"""Method to show input controls."""
		tp = '<input type="submit" name="%s" value="%s" style="%s">\n'
		if REQUEST is not None:
			seq = int(REQUEST.get('seq', self._numb))
			if seq > 0:
				s1 = tp % ('se_head', '|<', style)
				s2 = tp % ('se_prev', '<<', style)
			else:
				s1 = tp % ('', '&nbsp;&nbsp;', style)
				s2 = tp % ('', '&nbsp;&nbsp;', style)
			if seq < self._nmax:
				s3 = tp % ('se_next', '>>', style)
				s4 = tp % ('se_tail', '>|', style)
			else:
				s3 = tp % ('', '&nbsp;&nbsp;', style)
				s4 = tp % ('', '&nbsp;&nbsp;', style)
			return s1 + s2 + s3 + s4

	security.declareProtected('Add ZShogi', 'standardProperties')
	def standardProperties(self, REQUEST=None):
		"""Method to get ZShogi properties"""
		if REQUEST is not None:
			aq = getattr(self, 'aq_parent', None)
			if aq:
				items = aq.propertyItems()
			else:
				items = self.propertyItems()
			list = []
			if not REQUEST.form:
				for item in items:
					k = item[0]
					if k[:3] == 'sd_':
						v = self.getProperty(k, '')
						c = item[1]
						list.append((k, (c, v)))
				REQUEST['sd_title'] = self.title
			elif REQUEST.has_key('se_update'):
				for item in items:
					k = item[0]
					if k[:3] == 'sd_':
						v = REQUEST.get(k, '')
						if self.hasProperty(k):
							self.manage_changeProperties({k:v})
						else:
							self.manage_addProperty(k, v, 'string')
						c = item[1]
						list.append((k, (c, v)))
				if REQUEST.has_key('sd_title'):
					self.title = REQUEST['sd_title']
			else:
				for item in items:
					k = item[0]
					if k[:3] == 'sd_':
						v = REQUEST.get(k, '')
						c = item[1]
						list.append((k, (c, v)))
			return list

#	def getStandardProperty(self, REQUEST):
#		items = REQUEST.PARENTS[0].propertyItems()
#		list = []
#		for item in items:
#			k = item[0]
#			if k[:3] == 'sd_' and k != 'sd_title':
#				v = self.getProperty(k, '')
#				list.append((k, v))
#		return list

InitializeClass(ZShogi)

