# vim: set fileencoding=ascii ts=4 sw=4 sts=0 :
# Copyright (C) 2004, Shu KONNO <owa@bg.wakwak.com> All Rights Reserved.
import re

def comp_lu(a, b):
	""" method to compare positions between two pieces """ 
	if a[1] > b[1]:
		return 1
	if a[1] < b[1]:
		return -1
	if a[0] > b[0]:
		return -1
	if a[0] < b[0]:
		return 1
	return 0

class CSpace(object):
	def __init__(self):
		self.posdict = {}
		#self.blacket0={'K':0, 'R':0,'B':0,'G':0,'S':0,'N':0,'L':0,'P':0}
		self.blacket1={'K':0, 'R':0,'B':0,'G':0,'S':0,'N':0,'L':0,'P':0}
		self.blacket2={'K':0, 'R':0,'B':0,'G':0,'S':0,'N':0,'L':0,'P':0}

	def what(self, pos=''):
		return self.posdict.get(pos, '')

	def roll(self, pos):
		if pos and self.posdict.has_key(pos):
			piece = self.posdict[pos]
			if piece[0] == '1':
				piece = '2' + piece[1:]
			else:
				piece = '1' + piece[1:]
			self.posdict[pos] = piece

	def plus(self, pos):
		if pos and self.posdict.has_key(pos):
			piece = self.posdict[pos]
			# docheck!! in K,G 
			if piece[1] == '+':
				piece = piece[0] + piece[2]
			else:
				piece = piece[0] + '+' + piece[1]
			self.posdict[pos] = piece

	def drop(self, piece, pos=None, id=0):
		if id == 0:
			if self.posdict.has_key(pos):
				cap = self.posdict[pos]
				if piece[0] == '1':
					self.blacket1[cap[-1]] += 1
				else:
					self.blacket2[cap[-1]] += 1
			self.posdict[pos] = piece
		elif id == 1:
			self.blacket1[piece[-1]] += 1
		elif id == 2:
			self.blacket2[piece[-1]] += 1

	def take(self, pick):
		s = None
		if pick[2]:
			pos = pick[2]
			s = self.posdict.get(pos, None)
			if s:
				del self.posdict[pos]
		else:
			bw = pick[0]
			piece = pick[1]
			if bw == '1':
				n = self.blacket1[piece]
				if n > 0:
					self.blacket1[piece] = n - 1
					s = bw + piece
			elif bw == '2':
				n = self.blacket2[piece]
				if n > 0:
					self.blacket2[piece] = n - 1
					s = bw + piece
		return s

	def move(self, op):
		try:
			dst, pre, out = op.get(self)
			if out:
				self._capt(dst)
			self._move(dst, pre)
		except:
			print 'exception: CSpace::move=', dst, pre, out

	def undo(self, op):
		dst, pre, out = op.get(self)
		try:
			bw = self._unmove(dst, pre)
			if out:
				self._uncapt(dst, bw, out)
		except:
			print 'exception: CSpace::undo=', dst, pre, out

	def _capt(self, dst):
		pos = dst[:2]
		piece = self.posdict[pos]
		del self.posdict[pos]
		if piece[0] == '1':
			self.blacket2[piece[-1]] += 1
		else:
			self.blacket1[piece[-1]] += 1

	def _uncapt(self, dst, bw, out):
		if bw == '1':
			self.blacket1[out[-1]] -= 1
			self.posdict[dst] = '2' + out
		else:
			self.blacket2[out[-1]] -= 1
			self.posdict[dst] = '1' + out

	def _move(self, dst, src):
		if src[1] >= '1' and src[1] <= '9':
			key = src[:2]
			piece = self.posdict[key]
			del self.posdict[key]
		else:
			piece = src
			if src[0] == '1':
				self.blacket1[src[1]] -= 1
			else:
				self.blacket2[src[1]] -= 1

		if src[-1] == '+':
			self.posdict[dst] = piece[0] + '+' + piece[-1]
		else:
			self.posdict[dst] = piece

	def _unmove(self, dst, pre):
		piece = self.posdict[dst]
		del self.posdict[dst]
		if pre[1] >= '1' and pre[1] <= '9':
			if pre[-1] == '+':
				self.posdict[pre[:2]] = piece[0] + piece[-1]
			else:
				self.posdict[pre[:2]] = piece
		else:
			if pre[0] == '1':
				self.blacket1[pre[1]] += 1
			else:
				self.blacket2[pre[1]] += 1
		return piece[0]

	def putDiagram(self, diag=''):
		diag = re.sub('\s', '', diag)
		side = '1'
		for s in diag.split(','):
			#s = s.strip()
			if s:
				if s[0] == '1' or s[0] == '2':
					side = s[0]
					s = s[1:]
				plus = ''
				if s[0] == '+':
					plus = '+'
					s = s[1:]
				type = s[0]
				if len(s) == 1:
					if side == '1':
						self.blacket1[type] = 1
					elif side == '2':
						self.blacket2[type] = 1
				elif len(s) > 2 and s[1] == '*':
					n = int(s[2:])
					if side == '1':
						self.blacket1[type] = n
					elif side == '2':
						self.blacket2[type] = n
				elif len(s) >= 3:
					self.posdict[s[1:3]] = side + plus + type

	def getDiagram(self):
		""" method to get a diagram script """ 
		poslist = self.posdict.keys()
		poslist.sort(comp_lu)
		s1 = '1'
		s2 = '2'
		for pos in poslist:
			c = self.posdict[pos]
			if c[0] == '1':
				s1 += c[1:] + pos + ','
			if c[0] == '2':
				s2 += c[1:] + pos + ','
		for c in self.blacket1.keys():
			n = self.blacket1[c]
			if n == 1:
				s1 += c + ','
			elif n > 1:
				s1 += '%c*%d,' % (c, n)
		for c in self.blacket2.keys():
			n = self.blacket2[c]
			if n == 1:
				s2 += c + ','
			elif n > 1:
				s2 += '%c*%d,' % (c, n)
		s = ''
		if len(s2) > 1:
			s += s2
			s += '\n'
		if len(s1) > 1:
			s += s1
		if len(s) > 0:
			s = s[:-1]
		return s

	def getStays(self, bw=1):
		""" method to get a list on the board """ 
		poslist = self.posdict.keys()
		poslist.sort(comp_lu)
		s = []
		for pos in poslist:
			c = self.posdict[pos]
			if c and int(c[0]) == bw:
				s.append(c[1:] + pos)
		return s

	def getFrees(self, bw=1):
		""" method to get a list on the blacket """ 
		blacket = (bw == 1 and self.blacket1) or self.blacket2
		s = []
		for c in 'KRBGSNLP':
			n = blacket[c]
			if n > 0:
				s.append((c, n))
		return s

