# vim: set fileencoding=ascii ts=4 sw=4 sts=0 :
# Copyright (C) 2004, Shu KONNO <owa@bg.wakwak.com> All Rights Reserved.
from __init__ import opening_stage_figure
from ViewSpace import ViewSpace
from CSpace import CSpace
from CScript import CScript
import string

class COpenSpace(CSpace):
	def __init__(self, diagram):
		CSpace.__init__(self)
		diag = opening_stage_figure.get(diagram, diagram)
		CSpace.putDiagram(self, diag)

	def move(self, dst, pick, id=0):
		s = CSpace.take(self, pick)
		if s:
			CSpace.drop(self, s, dst, id)

	def plus(self, pick):
		CSpace.plus(self, pick[2])

	def roll(self, pick):
		if pick[2]:
			CSpace.roll(self, pick[2])
		else:
			s = CSpace.take(self, pick)
			if s:
				CSpace.drop(self, s, id=3-int(pick[0]))

class CMakeSpace(CScript):
	def __init__(self, diagram=None, scripts=None):
		CSpace.__init__(self)
		CScript.__init__(self)
		diag = opening_stage_figure.get(diagram, diagram)
		CSpace.putDiagram(self, diag)
		self.scripts = []
		if scripts:
			self.scripts = scripts.split(',')

	def getScripts(self):
		return string.join(self.scripts, ',')

	def getCount(self):
		return len(self.scripts)

	def move(self, dst, pick):
		op = CScript
		op.setp(self, dst, pick)
		CSpace.move(self, op)
		self.scripts.append(op.gets(self))

	def undo(self):
		if self.scripts:
			op = CScript
			op.sets(self, self.scripts[-1])
			CSpace.undo(self, op)
			self.scripts = self.scripts[:-1]

	def promote(self):
		if self.scripts:
			s = self.scripts[-1]
			op = CScript
			op.sets(self, s)
			CSpace.undo(self, op)

			self.scripts = self.scripts[:-1]
			op.sets(self, s)
			op.plus(self)
			CSpace.move(self, op)
			s = op.gets(self)
			self.scripts.append(s)

class CPlaySpace(CScript):
	def __init__(self, diagram='', scripts='', numb=0):
		CSpace.__init__(self)
		CScript.__init__(self)
		diag = opening_stage_figure.get(diagram, diagram)
		CSpace.putDiagram(self, diag)
		self.sequence = numb
		self.scripts = []
		if scripts:
			self.scripts = scripts.split(',')

	def getScripts(self):
		return string.join(self.scripts, ',')

	def getSequence(self):
		return self.sequence

	def getCount(self):
		return len(self.scrips)

	def next(self, n=0):
		s = None
		for s in self.scripts[self.sequence:]:
			s = self.scripts[self.sequence]
			op = CScript
			op.sets(self, s)
			CSpace.move(self, op)
			self.sequence += 1
			n -= 1
			if n == 0:
				break
		return s

	def prev(self, n=0):
		s = None
		for s in self.scripts[:self.sequence]:
			s = self.scripts[self.sequence - 1]
			op = CScript
			op.sets(self, s)
			CSpace.undo(self, op)
			self.sequence -= 1
			n -= 1
			if n == 0:
				break
		return s

class VCOpenSpace(ViewSpace, COpenSpace):
	def __init__(self, diagram='', size=0):
		ViewSpace.__init__(self, size=size)
		COpenSpace.__init__(self, diagram)

class VCMakeSpace(ViewSpace, CMakeSpace):
	def __init__(self, diagram='', scripts='', side='1', size=0):
		ViewSpace.__init__(self, size=size)
		CMakeSpace.__init__(self, diagram, scripts)

class VCPlaySpace(ViewSpace, CPlaySpace):
	def __init__(self, diagram='', scripts='', numb=0, side='1', size=0):
		ViewSpace.__init__(self, size=size)
		CPlaySpace.__init__(self, diagram, scripts, numb=numb)

